// Graph class
// Copyright (c) 2001 Tsutomu Saito All Rights Reserved

#ifndef HEADER_ONE_GRAPH
#define HEADER_ONE_GRAPH

#pragma warning(disable : 4786 4503) // for VC++
#include <vector>
#include <list>
#include <map>
#include <algorithm>
#include <iterator>
using namespace std;

template <typename Node,typename Arc>
class graph {
public:
	class node_tag;
	class arc_tag;
	class node_iterator;
	class arc_iterator;
	struct less_node_iterator;
	struct less_arc_iterator;
	typedef Node node_value_type;
	typedef Arc arc_value_type;
	typedef Node* node_pointer;
	typedef Arc* arc_pointer;
	typedef Node& node_reference;
	typedef Arc& arc_reference;
	typedef list<node_tag*> node_list;
	typedef list<arc_tag*> arc_list;
	typedef node_list::difference_type difference_type;
	typedef node_list::iterator node_list_iterator;
	typedef arc_list::iterator arc_list_iterator;
	typedef node_list::const_iterator node_list_const_iterator;
	typedef arc_list::const_iterator arc_list_const_iterator;
	struct less_node_list_iterator : binary_function<node_list_iterator,
		node_list_iterator, bool> {
	bool operator()(const node_list_iterator& i1,
		const node_list_iterator& i2)const{return *i1 < *i2;}
	};
	struct less_arc_list_iterator : binary_function<arc_list_iterator,
		arc_list_iterator, bool> {
	bool operator()(const arc_list_iterator& i1,
		const arc_list_iterator& i2)const{return *i1 < *i2;}
	};
	typedef map<node_list_iterator,arc_list_iterator,less_node_list_iterator> connect;
	typedef connect::iterator connect_iterator;
	class node_tag {
		friend graph;
		friend node_iterator;
		mutable int			m_temp;
		Node				m_node;
		connect				m_inarc;
		connect				m_outarc;
		node_tag(){}
		node_tag(const Node& n) : m_node(n) {}
		~node_tag(){}
	public:
		Node& node(){return m_node;}
		bool insert(bool bOut,node_list_iterator nit,arc_list_iterator ait)
			{return bOut ? insert_out(nit,ait) : insert_in(nit,ait);}
		bool insert_in(node_list_iterator nit,arc_list_iterator ait)
			{connect_iterator cit = m_inarc.find(nit);
			if (cit != m_inarc.end()) return false;
			m_inarc[nit] = ait; return true;}
		bool insert_out(node_list_iterator nit,arc_list_iterator ait)
			{connect_iterator cit = m_outarc.find(nit);
			if (cit != m_outarc.end()) return false;
			m_outarc[nit] = ait; return true;}
		bool erase(bool bOut,node_list_iterator nit)
			{return bOut ? erase_out(nit) : erase_in(nit);}
		bool erase_in(node_list_iterator nit)
			{return m_inarc.erase(nit) > 0;}
		bool erase_out(node_list_iterator nit)
			{return m_outarc.erase(nit) > 0;}
		int& temp()const{return m_temp;}
	};
	class arc_tag {
		friend graph;
		friend arc_iterator;
		Arc					m_arc;
		node_list_iterator	m_left,m_right;
		arc_tag(){}
		arc_tag(const Arc& a,const node_list_iterator& l,
			const node_list_iterator& r) : m_arc(a),m_left(l),m_right(r) {}
		~arc_tag(){}
	public:
		Arc& arc(){return m_arc;}
		void erase_node(bool bDir){(*m_left)->erase(bDir,m_right);
					(*m_right)->erase_in(m_left);}
		node_list_iterator& left(){return m_left;}
		node_list_iterator& right(){return m_right;}
	};
	class node_iterator : public iterator<bidirectional_iterator_tag,Node,difference_type> {
		friend graph;
		friend less_node_iterator;
		node_list_iterator	m_iter;
	public:
		node_iterator(){}
		node_iterator(node_list_iterator it) : m_iter(it) {}
		node_iterator(connect_iterator it) : m_iter(it->first) {}
		const Node& operator*()const{return (*m_iter)->node();}
		const Node* operator->()const{return &(*m_iter)->node();}
		Node& operator*(){return (*m_iter)->node();}
		Node* operator->(){return &(*m_iter)->node();}
		node_iterator& operator++(){++m_iter; return *this;}
		node_iterator operator++(int){node_iterator i = *this; ++m_iter; return i;}
		node_iterator& operator--(){--m_iter; return *this;}
		node_iterator operator--(int){node_iterator i = *this; --m_iter; return i;}
		bool operator==(const node_iterator& it)const
			{return m_iter == it.m_iter;}
		bool operator!=(const node_iterator& it)const
			{return m_iter != it.m_iter;}
		connect_iterator ibegin(){return (*m_iter)->m_inarc.begin();}
		connect_iterator obegin(){return (*m_iter)->m_outarc.begin();}
		connect_iterator begin(){return ibegin();}
		connect_iterator iend(){return (*m_iter)->m_inarc.end();}
		connect_iterator oend(){return (*m_iter)->m_outarc.end();}
		connect_iterator end(){return iend();}
		connect::size_type isize()const{return (*m_iter)->m_inarc.size();}
		connect::size_type osize()const{return (*m_iter)->m_outarc.size();}
		connect::size_type size()const{return isize();}
		operator node_list_iterator(){return m_iter;}
		int& temp()const{return (*m_iter)->m_temp;}
	};
	class arc_iterator : public iterator<bidirectional_iterator_tag,Arc,difference_type> {
		friend graph;
		friend less_arc_iterator;
		arc_list_iterator	m_iter;
	public:
		arc_iterator(){}
		arc_iterator(arc_list_iterator it) : m_iter(it) {}
		arc_iterator(connect_iterator it) : m_iter(it->second) {}
		const Arc& operator*()const{return (*m_iter)->arc();}
		const Arc* operator->()const{return &(*m_iter)->arc();}
		Arc& operator*(){return (*m_iter)->arc();}
		Arc* operator->(){return &(*m_iter)->arc();}
		arc_iterator& operator++(){++m_iter; return *this;}
		arc_iterator operator++(int){arc_iterator i = *this; ++m_iter; return i;}
		arc_iterator& operator--(){--m_iter; return *this;}
		arc_iterator operator--(int){arc_iterator i = *this; --m_iter; return i;}
		bool operator==(const arc_iterator& it)const
			{return m_iter == it.m_iter;}
		bool operator!=(const arc_iterator& it)const
			{return m_iter != it.m_iter;}
		node_iterator left(){return (*m_iter)->m_left;}
		node_iterator right(){return (*m_iter)->m_right;}
		operator arc_list_iterator(){return m_iter;}
	};
	struct less_node_iterator : binary_function<node_iterator,
		node_iterator, bool> {
	bool operator()(const node_iterator& i1,
		const node_iterator& i2)const{return *i1.m_iter < *i2.m_iter;}
	};
	struct less_arc_iterator : binary_function<arc_iterator,
		arc_iterator, bool> {
	bool operator()(const arc_iterator& i1,
		const arc_iterator& i2)const{return *i1.m_iter < *i2.m_iter;}
	};
private:
	bool		m_bDir;		// LOtǂ
	node_list	m_lnode;	// Node Xg
	arc_list	m_larc;		// Arc Xg
public:
	explicit graph(bool bDir = true){m_bDir = bDir;}
	// const ̈s\!!
	graph& operator=(const graph& g){
		if (this != &g) {
			node_list_const_iterator	nit;
			arc_list_const_iterator		ait;
			vector<node_iterator>		v(g.nsize());
			int							i;
			m_bDir = g.m_bDir;
			for (i=0, nit=g.m_lnode.begin();nit!=g.m_lnode.end();++nit, ++i)
				(*nit)->temp() = i;
			for (i=0, nit=g.m_lnode.begin();nit!=g.m_lnode.end();++nit, ++i)
				v[i] = add_node((*nit)->m_node);
			for (ait=g.m_larc.begin();ait!=g.m_larc.end();++ait) {
				const arc_tag& a = **ait;
				add_arc(a.m_arc,v[(*a.m_left)->temp()],v[(*a.m_right)->temp()]);
			}
		}
		return *this;
	}
	graph(const graph& g){*this = g;}
	void clear(){
		node_list_iterator	nit;
		arc_list_iterator	ait;
		for (ait=m_larc.begin();ait!=m_larc.end();++ait)
			delete *ait;
		for (nit=m_lnode.begin();nit!=m_lnode.end();++nit)
			delete *nit;
		m_larc.clear();
		m_lnode.clear();
	}
	~graph(){clear();}
	bool is_directed()const{return m_bDir;}
	node_iterator nbegin(){return m_lnode.begin();}
	node_iterator nend(){return m_lnode.end();}
	arc_iterator abegin(){return m_larc.begin();}
	arc_iterator aend(){return m_larc.end();}
	node_list::size_type nsize()const{return m_lnode.size();}
	arc_list::size_type asize()const{return m_larc.size();}
	bool has_path(const node_iterator& l,const node_iterator& r){
		node_iterator nit; connect_iterator cit;
		for (nit=nbegin();nit!=nend();++nit) nit.temp() = 0;
		list<node_iterator> s,t; s.push_back(l); l.temp() = 1;
		list<node_iterator>::iterator lit;
		while (!s.empty() && !r.temp()) {
			t.clear();
			for (lit=s.begin();lit!=s.end();++lit) {
				for (cit=lit->obegin();cit!=lit->oend();++cit) {
					nit = cit->first;
					if (!nit.temp()) {t.push_back(nit); nit.temp() = 1;}
				}
			}
			s = t;
		}
		return r.temp() != 0;
	}
	node_iterator add_node(const Node& n)
		{return m_lnode.insert(m_lnode.end(),new node_tag(n));}
	pair<arc_iterator,bool> add_arc(const Arc& a,const node_iterator& l,
		const node_iterator& r){arc_list_iterator
		it = m_larc.insert(m_larc.end(),new arc_tag(a,l.m_iter,r.m_iter));
		bool b = (*r.m_iter)->insert_in(l.m_iter,it);
		if (b) (*l.m_iter)->insert(m_bDir,r.m_iter,it);
		else m_larc.erase(it);
		return make_pair(static_cast<arc_iterator>(it),b);}
	void del_node(node_iterator nit){connect_iterator cit;
		node_list_iterator it = nit.m_iter;
		while ((cit = nit.ibegin()) != nit.iend()) {
			arc_list_iterator ait = cit->second;
			(*ait)->erase_node(m_bDir); delete *ait; m_larc.erase(ait);}
		while ((cit = nit.obegin()) != nit.oend()) {
			arc_list_iterator ait = cit->second;
			(*ait)->erase_node(m_bDir); delete *ait; m_larc.erase(ait);}
		delete *it; m_lnode.erase(it);}
	void del_arc(arc_iterator ait)
		{arc_list_iterator it = ait.m_iter;
		(*it)->erase_node(m_bDir); delete *it; m_larc.erase(it);}
	arc_iterator connect_arc(node_iterator nit1,node_iterator nit2)
		{connect& c = (*nit2.m_iter)->m_inarc;
		connect_iterator cit = c.find(nit1.m_iter);
		return arc_iterator(cit == c.end() ? m_larc.end() : cit->second);}
	bool is_valid(){node_iterator nit; arc_iterator ait; connect_iterator cit;
		for (nit=nbegin();nit!=nend();++nit) {
			for (cit=nit.ibegin();cit!=nit.iend();++cit) {
				ait = cit->second;
				if (find(m_larc.begin(),m_larc.end(),*ait.m_iter) == m_larc.end()) return false;
				if (m_bDir && ait.right() != nit) return false;
				connect& cn = (*cit->first)->m_outarc;
				if (m_bDir && cn.find(nit) == cn.end()) return false;
			}
			for (cit=nit.obegin();cit!=nit.oend();++cit) {
				ait = cit->second;
				if (find(m_larc.begin(),m_larc.end(),*ait.m_iter) == m_larc.end()) return false;
				if (m_bDir && ait.left() != nit) return false;
				connect& cn = (*cit->first)->m_inarc;
				if (cn.find(nit) == cn.end()) return false;
			}
		}
		for (ait=abegin();ait!=aend();++ait) {
			nit = ait.left();
			if (find(m_lnode.begin(),m_lnode.end(),*nit.m_iter) == m_lnode.end()) return false;
			if (m_bDir) {
				for (cit=nit.obegin();cit!=nit.oend();++cit)
					if (ait.m_iter == cit->second) break;
				if (cit == nit.oend()) return false;
			} else {
				for (cit=nit.ibegin();cit!=nit.iend();++cit)
					if (ait.m_iter == cit->second) break;
				if (cit == nit.iend()) return false;
			}
			nit = ait.right();
			if (find(m_lnode.begin(),m_lnode.end(),*nit.m_iter) == m_lnode.end()) return false;
			for (cit=nit.ibegin();cit!=nit.iend();++cit)
				if (ait.m_iter == cit->second) break;
			if (cit == nit.iend()) return false;
		}
		return true;}
};

#endif // HEADER_ONE_GRAPH
